#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.views import View

from django.utils.translation import ugettext_lazy as _

from django.shortcuts import render, redirect, get_object_or_404
from django.urls import reverse_lazy
from django.contrib import messages

from lostplaces.common import get_all_subclasses
from lostplaces.views.base_views import IsAuthenticatedMixin
from lostplaces.models.models import Explorer
from lostplaces.models.place import Place, PlaceAsset
from lostplaces.forms import ExplorerChangeForm, ExplorerUserChangeForm

class ExplorerProfileView(IsAuthenticatedMixin, View):
    def get(self, request, explorer_id):
        explorer = get_object_or_404(Explorer, pk=explorer_id)
        place_list = Place.objects.filter(submitted_by=explorer)
        place_count = place_list.count()

        context={
            'explorer': explorer,
            'place_count': place_count,
            'place_list': place_list,
            'assets': {}
        }

        asset_count = 0
        for subclass in get_all_subclasses(PlaceAsset): # kinda ugly, but PlaceAsset cannot be abstract according to django ORM
            objects = subclass.objects.filter(submitted_by=explorer)
            context['assets'][subclass.__name__.lower()+'s'] = objects
            asset_count += objects.count()
        
        context['asset_count'] = asset_count
                
        return render(
            request=request,
            template_name='explorer/profile.html',
            context=context
        )

class ExplorerProfileUpdateView(IsAuthenticatedMixin, View):
    success_message = ''
    permission_denied_message = ''

    def get(self, request, *args, **kwargs):
        context = {
            'explorer_user_change_form': ExplorerUserChangeForm(instance=request.user),
            'explorer_change_form': ExplorerChangeForm(instance=request.user.explorer)
        }
        if request.user.explorer.profile_image:
            context['explorer_image_url'] = request.user.explorer.profile_image.url
        return render(request, 'explorer/profile_update.html', context)

    def post(self, request, *args, **kwargs):
        explorer_user_change_form = ExplorerUserChangeForm(
            request.POST,
            instance=request.user
            )
        explorer_change_form = ExplorerChangeForm(
            request.POST,
            request.FILES,
            instance=request.user.explorer
        )

        if explorer_change_form.is_valid() and explorer_user_change_form.is_valid():
            explorer_user_change_form.save()
            explorer_change_form.save()

            messages.success(
                self.request,
                _('Successfully updated Explorer profile')
            )
        else:
            # Usually the browser should have checked the form before sending.
            messages.error(
                self.request,
                _('Please fill in all required fields.')
            )
        return redirect(reverse_lazy('explorer_profile_update'))
    